/* axconf.c
 *
 * Copyright (C) 2001 RedHat, Inc.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 *
 */
/*
 * Modified for SIKIGAMI 2001,2002 Axe,Inc.
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <unistd.h>
#include <malloc.h>
#include "axconf.h"

#define	STREQ(a,b)		((a)==(b)||(NULL!=(a)&&NULL!=(b)&&0==strcmp(a,b)))

static int
check_for_bool_string(char *str)
{
    if (strcasecmp(str, "off") == 0
     || strcasecmp(str, "false") == 0
     || strncasecmp(str, "no", 2) == 0)	// also handles "none"
	return (0);

    if (strcasecmp(str, "on") == 0
     || strcasecmp(str, "true") == 0
     || strcasecmp(str, "yes") == 0)
	return (1);

    // not a special bool case, so just return the number
    return (atoi(str));
}

static char *
trimspace( char * buf )
{
 char * sp, * ep;

	sp = buf;
	ep = buf + strlen(buf)-1;

	while( ' ' == *sp || '\t' == *sp ) {
		sp++;
	}

	while( sp<ep && ( ' ' == *ep || '\t' == *ep ) ) {
		ep--;
	}

	*(ep+1) = 0x00;

	return sp;
}

static char *
parse_section( char * buf )
{
 char * sp, * ep;

	if ( '[' != *buf ) {
		return NULL;
	}

	sp = buf+1;
	ep = strchr( sp, ']' );

	if ( !ep || ep[1] ) {
		return NULL;
	}

	ep[0] = 0x00;

	return trimspace( sp );
}

static char *
parse_value( char * buf )
{
 char * valtop;

	valtop = strchr( buf, '=' );
	if ( ! valtop ) {
		return NULL;
	}

	(*valtop) = 0x00;
	valtop++;

	return trimspace( valtop );
}

/*
 * AxConfigEntry Class
 */
void
AxConfigEntry_init( AxConfigEntry * entry )
{
	entry->name = NULL;
	entry->value = NULL;
}
void
AxConfigEntry_clear( AxConfigEntry * entry )
{
	if ( entry->name ) {
		g_free( entry->name );
	}
	
	if ( entry->value ) {
		g_free( entry->value );
	}
}
void
AxConfigEntry_init_valued( AxConfigEntry * entry, char * name, char * value )
{
 char * newName;
 char * newValue;

	if ( ! name  ) name  = "";
	if ( ! value ) value = "";

  newName  = (char *)g_malloc( strlen(name) +1 );
  newValue = (char *)g_malloc( strlen(value)+1 );

	strcpy( newName,  name  );
	strcpy( newValue, value );

	entry->name  = newName;
	entry->value = newValue;
}
void
AxConfigEntry_set( AxConfigEntry * entry, char * value )
{
 char * ptr;

	if ( ! value ) value = "";
 
	ptr = (char *)g_malloc(sizeof(char)*(strlen(value)+1));

	strcpy( ptr, value );

	if ( entry->value ) {
		g_free( entry->value );
	}

	entry->value = ptr;
}

/*
 * AxConfigSection Class
 */
void
AxConfigSection_init( AxConfigSection * section )
{
	section->entries = NULL;
	section->name = NULL;
}
void
AxConfigSection_init_valued( AxConfigSection * section, char * name )
{
 char * ptr;

	if ( ! name ) name = "";

	AxConfigSection_init( section );
	
	ptr = (char *)g_malloc(sizeof(char)*(strlen(name)+1));
	strcpy( ptr, name );

	section->name = ptr;
}
void
AxConfigSection_clear( AxConfigSection * section )
{
 GSList * entry;

	entry = section->entries;
	while( entry ) {
		AxConfigEntry_clear( (AxConfigEntry*)entry->data );
		g_free( entry->data );
		entry->data = NULL;
		entry = g_slist_next( entry );
	}
	g_slist_free( entry );
	g_free( section->name );

	section->entries = NULL;
	section->name    = NULL;
}
AxConfigEntry *
AxConfigSection_at( AxConfigSection * section, char * name )
{
 GSList * list;
 AxConfigEntry * entry = NULL;

	if ( ! section ) {
		return NULL;
	}

	list = section->entries;
	while( NULL != list ) {
		entry = (AxConfigEntry*)list->data;
		if ( NULL != entry && 0==strcasecmp(name,entry->name) ) {
			break;
		}
		list = g_slist_next( list );
	}

	return ( list ? entry : NULL );
}
char *
AxConfigSection_defget( AxConfigSection * section, char * name, char * def )
{
 AxConfigEntry * entry;

	entry = AxConfigSection_at( section, name );

  return ( entry ? entry->value : def );
}
char *
AxConfigSection_get( AxConfigSection * section, char * name )
{
	return AxConfigSection_defget( section, name, NULL );
}
void
AxConfigSection_set( AxConfigSection * section, char * name, char * value )
{
 AxConfigEntry * entry;

	entry = AxConfigSection_at( section, name );

	if ( ! entry && value && value[0] ) {
		entry = (AxConfigEntry*)g_malloc( sizeof(AxConfigEntry) );
		AxConfigEntry_init_valued( entry, name, value );
		AxConfigSection_append( section, entry );
	}
	else if ( entry ) {
		if ( value && value[0] ) {
		  AxConfigEntry_set( entry, value );
	  }
		else {
			section->entries = g_slist_remove( section->entries, (gpointer*)entry );
		  AxConfigEntry_clear( entry );
			g_free( entry );
		}
	}
}
void
AxConfigSection_quickset( AxConfigSection * section, char * name, char * value )
{
 AxConfigEntry * entry;

		entry = (AxConfigEntry*)g_malloc( sizeof(AxConfigEntry) );

		AxConfigEntry_init_valued( entry, name, value );

		AxConfigSection_append( section, entry );
}
void
AxConfigSection_append( AxConfigSection * section, AxConfigEntry * entry )
{
	if ( section->entries ) {
		section->entries = g_slist_append( section->entries, (gpointer)entry );
	}
	else {
		section->entries = g_slist_alloc( );
		section->entries->data = (gpointer)entry;
	}
}
void
AxConfigSection_dump( AxConfigSection *section, int fp )
{
 GSList * list = section->entries;
 AxConfigEntry * entry = NULL;

	while( list ) {
		entry = (AxConfigEntry*)list->data;

		if ( entry->name && entry->name[0] ) {
			write( fp, entry->name, strlen(entry->name) );
			write( fp, " = ", 3 );

			if ( entry->value && entry->value[0] ) {
				write( fp, entry->value, strlen(entry->value) );
			}

			write( fp, "\n", 1 );
		}
		list = g_slist_next( list );
	}
}

/*
 * AxConfCallbackInfo Class
 */
void
AxConfCallbackInfo_init( AxConfCallbackInfo * info, AxConfCallback * callback,
												 void * data, char * section, char * entry )
{
	info->callback = callback;
	info->userdata = data;
	info->section = section;
	info->entry = entry;
}

/*
 * AxConf Class
 */
void
AxConf_init_valued( AxConf * conf, char * filename )
{
	AxConf_init_with_mode( conf, filename, 0600 );
}
void
AxConf_init_with_mode( AxConf * conf, char * filename, int mode )
{
		AxConfigSection_init_valued( & conf->globalsection, "" );

    conf->create_mode = mode;
		conf->dirty = 0;
		conf->error = 0;

		conf->callbacks = NULL;
		conf->sections  = NULL;

		conf->filename = (char *)g_malloc( sizeof(char)*(strlen(filename)+1) );
		strcpy( conf->filename, filename );
		
    AxConf_load_file( conf );
}
void
AxConf_clear( AxConf * conf )
{
 GSList * list;

	AxConf_flush( conf );

	AxConfigSection_clear( & conf->globalsection );

	list = conf->sections;
	while( list ) {
		AxConfigSection_clear( (AxConfigSection*)list->data );
		list = g_slist_next( list );
	}

	g_slist_free( conf->sections );
	g_slist_free( conf->callbacks );

}
int AxConf_isok( AxConf * conf )
{
	return ! conf->error;
}
int AxConf_isclean( AxConf * conf )
{
	return AxConf_isok(conf) && ! conf->dirty;
}
void
AxConf_flush( AxConf * conf )
{
 int	fp;
 GSList * list;

  if (!conf->dirty || conf->error)
		return;
    
	fp = open( conf->filename, O_WRONLY|O_CREAT|O_TRUNC, conf->create_mode);

  if ( 1 > fp ) {
//		fprintf( stderr, "Can't write to config file:%s\n", strerror(errno) );
		conf->error = 1;
		return;
  }

  AxConfigSection_dump( &conf->globalsection, fp );

	list = conf->sections;

	while( list ) {
		AxConfigSection * section = (AxConfigSection*)list->data;
		write( fp, "[", 1 );
		write( fp, section->name, strlen( section->name ) );
		write( fp, "]\n", 2 );
		AxConfigSection_dump( section, fp );

		list = g_slist_next( list );
	}

	conf->dirty = 0;

	close( fp );
}
AxConfigSection *
AxConf_at( AxConf * conf, char * name )
{
 GSList * list;
 AxConfigSection * section;

	list = conf->sections;

	while( list ) {
		section = (AxConfigSection*)list->data;
		if ( section->name && 0 == strcasecmp(section->name,name) ) {
			return section;
		}

		list = g_slist_next( list );
	}

  return NULL;
}
int
AxConf_getint( AxConf * conf, char * section, char * entry, int def )
{
 char defstr[ 30 ];
 int	ret;

	sprintf( defstr, "%d", def );

	ret=check_for_bool_string( AxConf_defget( conf, section, entry, defstr ) );

	return ret;
}
char *
AxConf_get( AxConf * conf, char * section, char * entry )
{
	return AxConf_defget( conf, section, entry, NULL );
}
char *
AxConf_defget( AxConf * conf, char * section, char * entry, char * def )
{
 char * cache[5];
 AxConfigSection * found;
 int	cnt, idx;
 char	* value;

	memset( cache, 0x00, sizeof(cache));

	found = AxConf_at( conf, section );

	while( found ) {
		for( cnt=0; 5>cnt; cnt++ ) {
			if ( cache[cnt] && 0==strcmp(cache[cnt],found->name) ) {
				break;
			}
		}
		if ( 5 > cnt ) {
			break;
		}

		value = AxConfigSection_get( found, entry );
		if ( value ) {
			return value;
		}

		for( idx=0; 5>idx && cache[idx]; idx++ ) {
		}

		cache[idx] = found->name;

		if ( AxConfigSection_at( found, "Inherits" ) ) {
			found = AxConf_at( conf, AxConfigSection_get( found, "Inherits" ) );
		}
		else {
			found = NULL;
		}
	}

	return AxConfigSection_defget( &conf->globalsection, entry, def );
}
int
AxConf_fuzzy_getint_list( AxConf * conf, GSList * section, GSList * entstr, int def )
{
 char	defstr[ 30 ];
 int	ret;

	sprintf( defstr, "%d", def );
	
	ret=check_for_bool_string(AxConf_fuzzy_get_list(conf,section,entstr,defstr));

	return ret;
}
int
AxConf_fuzzy_getint( AxConf * conf, GSList * sectstr, char * entstr, int def )
{
 char	defstr[ 30 ];
 int	ret;

	sprintf( defstr, "%d", def );
	
	ret=check_for_bool_string(AxConf_fuzzy_get(conf,sectstr,entstr,defstr));

	return ret;
}
char *
AxConf_fuzzy_get_list( AxConf * conf, GSList *sectstr, GSList * entstr, char * def )
{
 GSList	* slist, * elist;
 char * value, * section, * entry;

	for( slist=sectstr; slist; slist=g_slist_next(slist) ) {

		for( elist=entstr; elist; elist=g_slist_next(elist) ) {

//			section = (char *)slist->data;
			section = ((AxConfigSection *)slist->data)->name;
			entry   = (char *)elist->data;

			value = AxConf_get( conf, section, entry );
			if ( value ) {
				return value;
			}
		}
	}

	return def;
}
char *
AxConf_fuzzy_get( AxConf * conf, GSList *section, char * entstr, char * def )
{
 GSList * entries;
 char	* value;

	entries = g_slist_alloc( );
	entries->data = (void *)entstr;

	value = AxConf_fuzzy_get_list( conf, section, entries, def );

	g_slist_free( entries );

	return value;
}

void
AxConf_setint( AxConf * conf, char * section, char * entry, int value )
{
 char	valueStr[ 30 ];

	sprintf( valueStr, "%d", value );

	AxConf_set( conf, section, entry, valueStr );
}
void
AxConf_set( AxConf * conf, char * section, char * entry, char * value )
{
 AxConfigSection * cnfsct;
// GSList * list;
 char	  * oldValue;

	cnfsct = AxConf_at( conf, section );

	if ( ! cnfsct ) {
		if ( ! value || ! value[0] ) {
			return;
		}
		cnfsct = (AxConfigSection*)g_malloc( sizeof(AxConfigSection) );
		AxConfigSection_init_valued( cnfsct, section );

		conf->sections = g_slist_append( conf->sections, (gpointer)cnfsct );
	}

	oldValue = AxConfigSection_defget( cnfsct, entry, "" );

	if ( ! value ) {
		value = "";
	}

	if ( strcmp( oldValue, value ) ) {
		AxConf_run_callback( conf, section, entry, oldValue, value );
	}

	AxConfigSection_set( cnfsct, entry, value );

  conf->dirty = 1;
}
void
AxConf_delete_section( AxConf * conf, char * section )
{
 AxConfigSection * cnfsct;

	cnfsct = AxConf_at( conf, section );

	if ( cnfsct ) {
		g_slist_remove( conf->sections, (gpointer*)cnfsct );

		AxConfigSection_clear( cnfsct );
		g_free( cnfsct );
	}

	conf->dirty = 1;
}
void
AxConf_setbool( AxConf * conf, void * data,
								char * str1, char * str2, char * str3, char str4 )
{
	*(int*)data = 1;
}
void
AxConf_load_file( AxConf * conf )
{
	AxConf_load_file_named( conf, conf->filename );
}
void
AxConf_load_file_named( AxConf * conf, char * filename )
{
 FILE *	fp;
 char buf[ 256 ], * strtop, * data;
 int	mode;
 AxConfigSection * section = & conf->globalsection;

	mode = 0;

	fp = fopen( filename, "r" );

	if ( ! fp ) {
//		fprintf( stderr, "Can't read config file %s:%s\n", filename, strerror(errno) );
		if ( ENOENT != errno ) {
			conf->error = 1;
		}
		return;
	}

	do {
		if ( NULL == fgets( buf, sizeof(buf), fp ) ) {
			break;
		}
		if ( buf[0] && '\n' == buf[ strlen(buf)-1 ] ) {
			buf[ strlen(buf)-1 ] = 0x00;
		}
		strtop = trimspace(buf);
		data = parse_section( strtop );

		if ( data ) {
			mode = 0;
			if ( ! data[0] ) {
				section = & conf->globalsection;
			}
			else {
				section = AxConf_at( conf, data );
				if ( ! section ) {
					section = (AxConfigSection*)g_malloc( sizeof(AxConfigSection) );
					AxConfigSection_init_valued( section, data );
					if ( conf->sections ) {
					  conf->sections = g_slist_append( conf->sections, (gpointer)section );
				  }
				  else {
					  conf->sections = g_slist_alloc( );
						conf->sections->data = (gpointer)section;
						conf->sections->next = NULL;
					}
				}
			}
		}
		else {
			data = parse_value( strtop );
			if ( ! data ) {
				data = "";
			}
			strtop = trimspace( strtop );
			if ( strtop && strtop[0] ) {
				if ( mode ) {
					AxConfigSection_quickset( section, strtop, data );
				}
				else {
					AxConfigSection_set( section, strtop, data );
				}
			}
		}
	} while( ! feof(fp) );

	fclose(fp);
}
void
AxConf_add_callback( AxConf * conf, AxConfCallback * callback,
										 void * data, char * section, char * entry )
{
 AxConfCallbackInfo * info;

	info = (AxConfCallbackInfo*)g_malloc( sizeof(AxConfCallbackInfo) );
	AxConfCallbackInfo_init( info, callback, data, section, entry );

	if ( conf->callbacks ) {
		conf->callbacks = g_slist_append( conf->callbacks, (gpointer)info );
	}
	else {
		conf->callbacks = g_slist_alloc( );
		conf->callbacks->data = (gpointer)info;
		conf->callbacks->next = NULL;
	}
}
void
AxConf_del_callback( AxConf * conf, AxConfCallback * callback,
										 void * data, char * section, char * entry )
{
 AxConfCallbackInfo * info;
 GSList * list;

	list = conf->callbacks;

	while( list ) {
		info = (AxConfCallbackInfo *)list->data;
		if (   callback == info->callback
				&& data == info->userdata
				&& STREQ( section, info->section )
				&& STREQ( entry,   info->entry ) ) {
			g_slist_remove( conf->callbacks, (void *)info );
			return;
		}
		list = g_slist_next( list );
	}
}
void
AxConf_run_callback( AxConf * conf,
										 char * section, char * entry,
										 char * oldvalue,char * newvalue )
{
 GSList	* list;
 AxConfCallbackInfo * info;

	list = conf->callbacks;

	while( list ) {
		info = (AxConfCallbackInfo*)list->data;
		if ( ! info->section || ! strcasecmp(info->section,section) ) {
			if ( ! info->entry || ! strcasecmp(info->entry,entry) ) {
				(*info->callback)( conf, info->userdata, section, entry, oldvalue, newvalue );
			}
		}
	}
}
void
AxConf_add_setbool( AxConf *conf, int *bool, char *section, char *entry )
{
	AxConf_add_callback( conf, (AxConfCallback*)AxConf_setbool, bool, section, entry );
}
void
AxConf_del_setbool( AxConf *conf, int *bool, char *section, char *entry )
{
	AxConf_del_callback( conf, (AxConfCallback*)AxConf_setbool, bool, section, entry );
}

