/* axconf.h
 *
 * Copyright (C) 2001 RedHat, Inc.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 *
 */
/*
 * Modified for SIKIGAMI 2001,2002 Axe,Inc.
 */

#ifndef __AXCONF_H__
#define __AXCONF_H__

#include <glib.h>

typedef struct _AxConfigEntry AxConfigEntry;

struct	_AxConfigEntry {
	char	* name;
	char	* value;
};

extern	void AxConfigEntry_init( AxConfigEntry * );
extern	void AxConfigEntry_init_valued( AxConfigEntry *, char *, char * );
extern	void AxConfigEntry_clear( AxConfigEntry * );
extern	void AxConfigEntry_set( AxConfigEntry *, char * );

typedef struct _AxConfigSection AxConfigSection;

struct	_AxConfigSection {
	GSList * entries;
  char * name;
};

extern	void AxConfigSection_init( AxConfigSection * );
extern	void AxConfigSection_init_valued( AxConfigSection *, char * );
extern	void AxConfigSection_clear( AxConfigSection * );
extern	AxConfigEntry * AxConfigSection_at( AxConfigSection *, char * );
extern	char * AxConfigSection_defget( AxConfigSection *, char *, char * );
extern	char * AxConfigSection_get( AxConfigSection *, char * );
extern	void AxConfigSection_set( AxConfigSection *, char *, char * );
extern	void AxConfigSection_quickset( AxConfigSection *, char *, char * );
extern	void AxConfigSection_dump( AxConfigSection *, int );
extern	void AxConfigSection_append( AxConfigSection*,AxConfigEntry*);

typedef struct _AxConf AxConf;

typedef void AxConfCallback(AxConf *, void *, char *, char *, char *, char * );

typedef	struct _AxConfCallbackInfo AxConfCallbackInfo;

struct _AxConfCallbackInfo {
	AxConfCallback	* callback;
	void * userdata;
	char * section;
	char * entry;
};

extern	void AxConfCallbackInfo_init( AxConfCallbackInfo *, AxConfCallback *, void *, char *, char * );

struct _AxConf {
	AxConfigSection	globalsection;
	GSList * sections;
	GSList * callbacks;
  int dirty;
  int error;
  int create_mode;

  char * filename;
};

extern	void AxConf_init_valued( AxConf *, char * );
extern	void AxConf_init_with_mode( AxConf *, char *, int );
extern	void AxConf_clear( AxConf * );

extern	int AxConf_isok( AxConf * );
extern	int AxConf_isclean( AxConf * );
extern	void AxConf_flush( AxConf * );

extern	AxConfigSection * AxConf_at( AxConf *, char * );
extern	int AxConf_getint( AxConf *, char *, char *, int );
extern	char * AxConf_get( AxConf *, char *, char * );
extern	char * AxConf_defget( AxConf *, char *, char *, char * );
extern	int AxConf_fuzzy_getint_list( AxConf *, GSList *, GSList *, int );
extern	int AxConf_fuzzy_getint( AxConf *, GSList *, char *, int );
extern	char * AxConf_fuzzy_get_list( AxConf *, GSList *, GSList *, char * );
extern	char * AxConf_fuzzy_get( AxConf *, GSList *, char *, char * );

extern	void AxConf_setint( AxConf *, char *, char *, int );
extern	void AxConf_set( AxConf *, char *, char *, char * );

extern	void AxConf_delete_section( AxConf *, char * );

extern	AxConfCallback setbool;

extern	void AxConf_load_file( AxConf * );
extern	void AxConf_load_file_named( AxConf *, char * );

extern	void AxConf_add_callback(AxConf*,AxConfCallback*,void*,char*,char*);
extern	void AxConf_del_callback(AxConf*,AxConfCallback*,void*,char*,char*);
extern	void AxConf_run_callback(AxConf*,char*,char*,char*,char*);

extern	void AxConf_add_setbool( AxConf*,int*,char*,char*);
extern	void AxConf_del_setbool( AxConf*,int*,char*,char*);
    
#endif 
