/* axmodem.c
 *
 * Copyright (C) 2001 RedHat, Inc.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */
/*
 * Modified for SIKIGAMI 2001,2002 Axe,Inc.
 */

#include "axmodem.h"
#include <sys/ioctl.h>
#include <stdio.h>
    	
typedef	struct _SpeedLookup SpeedLookup;
struct _SpeedLookup {
    int baud;
    speed_t speedt;
};


static SpeedLookup speeds[] = {
    {460800, B460800},
    {230400, B230400},
    {115200, B115200},
    { 57600, B57600},
    { 38400, B38400},
    { 19200, B19200},
    {  9600, B9600},
    {  4800, B4800},
    {  2400, B2400},
    {  1200, B1200},
    {   300, B300}
};


void
AxModemBase_init( AxModemBase * modem )
{
	modem->isbase = 1;
}
void
AxModemBase_init_valued( AxModemBase * modem, int _fd )
{
		modem->isbase = 1;
    modem->fd = _fd;
    AxModemBase_get_real_speed( modem );
}
void
AxModemBase_clear( AxModemBase * modem )
{
    // nothing needed
		AxStream_clear( (AxStream*)modem );
}
int
AxModemBase_isok( AxModemBase * modem )
{
	return AxStream_isok( (AxStream*)modem );
}
int
AxModemBase_speed( AxModemBase * modem )
{
    return modem->baud;
}
int
AxModemBase_get_real_speed( AxModemBase * modem )
{
    speed_t s;
		unsigned int i;
    
    if (!AxModemBase_isok(modem)) return 0;

    tcgetattr( modem->fd, &modem->t );
    s = cfgetospeed( &modem->t );
    for (i=0; i<sizeof(speeds)/sizeof(*speeds); i++) {
			if (speeds[i].speedt == s) {
	    	modem->baud = speeds[i].baud;
	    	break;
			}
    }

    return modem->baud;
}
void
AxModemBase_close( AxModemBase * modem )
{
    // no file open, no need to close it
}
int
AxModemBase_carrier( AxModemBase * modem )
{
    return 1;
}
void
AxModemBase_hangup( AxModemBase * modem)
{
    int i, oldbaud = modem->baud;
    
    if (modem->die_fast || !AxModemBase_isok(modem)) return;

    AxStream_drain( (AxStream*)modem );

    AxStream_write( (AxStream*)modem, "\r", 1 );
    for (i = 0; !AxStream_select((AxStream*)modem,200,1,0,0) && i < 10; i++)
			AxStream_write( (AxStream*)modem, "\r", 1 );

    AxStream_drain( (AxStream*)modem );

    // drop DTR for a while, if still online
    if (AxModemBase_carrier(modem)) {
			cfsetospeed( &modem->t, B0 );
			tcsetattr( modem->fd, TCSANOW, &modem->t );
			for (i = 0; AxModemBase_carrier(modem) && i < 10; i++)
	 		  usleep( 100 * 1000 );

			// raise DTR again, restoring the old baud rate
			AxModem_speed((AxModem*)modem,oldbaud);
    }
    
    if (AxModemBase_carrier(modem)) {
			// need to do +++ manual-disconnect stuff
			AxStream_write( (AxStream*)modem, "+++", 3 );
			usleep( 1500 * 1000 );
			AxStream_write( (AxStream*)modem, "ATH\r", 4 );
	
			for (i = 0; AxModemBase_carrier(modem) && i < 5; i++)
	    	usleep( 100 * 1000 );
    }
}
void
AxModem_init_valued( AxModem * modem, char * filename, int _baud )
{
	AxModemBase_init( (AxModemBase*)modem );
	modem->isbase = 0;
	
  modem->closing = 0;
  modem->baud = _baud;
  modem->die_fast = 0;

  modem->fd = open(filename, O_RDWR|O_NONBLOCK|O_NOCTTY);

  if (AxModemBase_isok((AxModemBase*)modem)) {
		AxModem_setup_modem( modem );
	}
}
void
AxModem_clear( AxModem * modem )
{
		if ( modem->isbase ) {
			AxModemBase_clear( (AxModemBase*)modem );
		}
		else {
	    close( modem->fd );
		}
}
int AxModem_isok( AxModem * modem )
{
	if ( modem->isbase ) {
		return AxModemBase_isok( (AxModemBase*)modem );
	}
	else {
		return AxStream_isok( (AxStream*)modem );
	}
}

int
AxModemBase_getspeed( AxModemBase * modem )
{
	return modem->baud;
}

void
AxModem_setup_modem( AxModem * modem )
{
		int i;

    if (!AxStream_isok((AxStream*)modem)) return;

    if (tcgetattr( modem->fd, &modem->t ) || tcgetattr( modem->fd, &modem->old_t ))
    {
			modem->closing = 1;
			return;
    }
    
    AxStream_drain( (AxStream*)modem );
    
    // set up the terminal characteristics.
    // see "man tcsetattr" for more information about these options.
    modem->t.c_iflag &= ~(BRKINT | ISTRIP | IUCLC | IXON | IXANY | IXOFF | IMAXBEL);
    modem->t.c_iflag |= (IGNBRK | IGNPAR);
    modem->t.c_oflag &= ~(OLCUC);
    modem->t.c_cflag &= ~(CSIZE | CSTOPB | PARENB | PARODD);
    modem->t.c_cflag |= (CS8 | CREAD | HUPCL | CRTSCTS | CLOCAL);
    modem->t.c_lflag &= ~(ISIG | XCASE | ECHO);
    tcsetattr( modem->fd, TCSANOW, &modem->t );
    
    // make sure we leave the modem in CLOCAL when we exit, so normal user
    // tasks can open the modem without using nonblocking.
    modem->old_t.c_cflag |= CLOCAL;
    
    // Send a few returns to make sure the modem is "good and zonked".
    if (cfgetospeed(&modem->t) != B0) {
			for( i=0; i<5; i++ ) {
	    	AxStream_write( (AxStream*)modem, "\r", 1 );
	    	usleep( 10 * 1000 );
			}
    }

    // Set the baud rate to 0 for half a second to drop DTR...
    cfsetispeed( &modem->t, B0 );
    cfsetospeed( &modem->t, B0 );
    cfmakeraw( &modem->t );
    tcsetattr( modem->fd, TCSANOW, &modem->t );
    if ( AxModem_carrier( modem ))
			usleep( 500 * 1000 );
    
    AxModem_speed(modem,modem->baud);
    usleep( 10 * 1000 );
    
    AxStream_drain((AxStream*)modem);
}
void
AxModem_close( AxModem * modem )
{
		if ( modem->isbase ) {
			AxModemBase_close( (AxModemBase*)modem );
			return;
		}

    if (modem->fd >= 0) {
			if (!modem->closing) {
	    	modem->closing = 1;
	    	AxModemBase_hangup( (AxModemBase*)modem );
			}
			modem->closing = 1;
			tcflush(modem->fd, TCIOFLUSH);
			tcsetattr(modem->fd, TCSANOW, &modem->old_t);
			close( modem->fd );
			modem->closing = 0;
    }
}

int
AxModem_speed(AxModem*modem,int _baud)
{
    speed_t s = B0;
		unsigned int i;

		if ( modem->isbase ) {
			return AxModemBase_speed( (AxModemBase*)modem );
		}
    modem->baud = 0;
    for ( i=0; i < sizeof(speeds) / sizeof(*speeds); i++) {
			if (speeds[i].baud <= _baud) {
	    	s = speeds[i].speedt;
	    	break;
			}
    }

    cfsetispeed( &modem->t, B0 ); // auto-match to output speed
    cfsetospeed( &modem->t, s );
    tcsetattr( modem->fd, TCSANOW, &modem->t );

    return AxModemBase_get_real_speed( (AxModemBase*)modem );
}


int
AxModem_getstatus( AxModem * modem )
{
    int status = 0;

    if (!AxStream_isok((AxStream*)modem)) return 0;
    ioctl(modem->fd, TIOCMGET, &status);
    return status;
}

int
AxModem_carrier( AxModem * modem )
{
		if ( modem->isbase ) {
			return AxModemBase_carrier( (AxModemBase*)modem );
		}

    return ( AxModem_getstatus(modem) & TIOCM_CD) ? 1 : 0;
}
