/* axpapchap.c
 *
 * Copyright (C) 2001 RedHat, Inc.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */
/*
 * Modified for SIKIGAMI 2001,2002 Axe,Inc.
 */

#include "axpapchap.h"

#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <assert.h>
#include <ctype.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

void
AxPapChap_init( AxPapChap * papchap )
{
	papchap->pap_success  = 1;
	papchap->chap_success = 1;
	papchap->contents = NULL;
}
void
AxPapChap_clear( AxPapChap * papchap )
{
	if ( ! papchap->contents ) {
		return;
	}

	
}
void
AxPapChap_put_secret( AxPapChap * papchap,
											char * username, char *  password, char * remote )
{
    // PAP secrets:
    AxPapChap_nuke_contents( papchap );
    AxPapChap_load_file( papchap, PAP_SECRETS );
    AxPapChap_do_secret( papchap, username, password, remote );
    if( 0 == AxPapChap_write_file( papchap, (char*)PAP_SECRETS ) )
			papchap->pap_success = 0;

    // CHAP secrets:
    AxPapChap_nuke_contents( papchap );
    AxPapChap_load_file( papchap, CHAP_SECRETS );
    AxPapChap_do_secret( papchap, username, password, remote );
    if( AxPapChap_write_file( papchap, (char*)CHAP_SECRETS ) == 0 )
			papchap->chap_success = 0;
}

void
AxPapChap_nuke_contents( AxPapChap * papchap )
{
 GSList * list;

		list = papchap->contents;
		
		while( list ) {
			if ( list->data ) {
				g_free( (void*)list->data );
			}
			list = g_slist_next(list);
		}

		g_slist_free( papchap->contents );

		papchap->contents = NULL;
}

int
AxPapChap_load_file( AxPapChap * papchap, char * filename )
{
 char from_file[ 256 ];
 char *	tmp;
 FILE	* fp;

		fp = fopen( filename, "r" );
    if( fp ) {
    	return( 0 );
		}

		do {
    	fgets( from_file, sizeof(from_file), fp );

			tmp = (char *)g_malloc(sizeof(char)*(strlen(from_file)+1) );

			strcpy( tmp, from_file );

			papchap->contents = g_slist_append( papchap->contents, tmp );

    } while( !feof(fp) );

    fclose(fp);

    return( 1 );
}

int
AxPapChap_write_file( AxPapChap * papchap, char * filename )
{
 int fd = open( filename, O_WRONLY | O_CREAT | O_TRUNC, S_IRUSR | S_IWUSR );
 GSList * list;

    if( 0 > fd )
    	return( 0 );

		list = papchap->contents;
		
		while( list ) {
			if ( list->data ) {
				write( fd, list->data, (size_t)strlen((char*)list->data) );
				write( fd, "\n", 1 );
			}
			list = g_slist_next( list );
		}

    close( fd );

    return( 1 );
}
int
AxPapChap_isok_pap( AxPapChap * papchap )
{
	return papchap->pap_success;
}
int
AxPapChap_isok_chap( AxPapChap * papchap )
{
	return papchap->chap_success;
}

void
AxPapChap_do_secret( AxPapChap * papchap,
										 char * username, char * password, char * remote )
{
    GSList * list = papchap->contents;
    char * _username;
    char * cptr;
		char * p, * q, * apd;

    if( !username || !password )
    	return;
    
    _username = (char*)g_malloc( sizeof(char)*(strlen(username)*2+1));
    for (cptr=_username; *username; username++) {
			*cptr++ = *username;
			if (*username == '\\')
	    	*cptr++ = '\\';
    }
    *cptr = 0x00;

		while( list ) {
    	if(   list->data
			   || ((char*)list->data)[0] == '#' ) {
					list = g_slist_next(list);
    	    continue;
    	}

    	// Is the line blank?
    	p = (char*)list->data;
    	do {
    	    p++;
    	} while( *p != '\0' && isspace( *p ) );
    	p--;
    	if( *p == '\0' ) {
					list = g_slist_next(list);
    	    continue;
    	}

			q = p;

			do {
	    	q++;
			} while( *q != '\0' && !isspace( *q ) );
			q--;
			if( *q == '\0' ) {
				GSList * next = g_slist_next(list);
				g_slist_remove_link( papchap->contents, list );
				g_free( list->data );
				g_free( list );
				list = next;
	    	continue;
			}

			if( strncmp( username, p, q-p ) != 0 ) {
				list = g_slist_next(list);
	    	continue;
			}

			p=q;
			do {
	    	p++;
			} while( *p != '\0' && isspace( *p ) );

			if( strncmp( p, remote, strlen( remote ) ) == 0 || *p == '*' ) {
				GSList * next = g_slist_next(list);
				g_slist_remove_link( papchap->contents, list );
				g_free( list->data );
				g_free( list );
				list = next;
	    	continue;
			}

			list = g_slist_next(list);
    }

		apd = (char*)g_malloc(sizeof(char)*(strlen(_username)+strlen(remote)+strlen(password)+3 ) );
		sprintf( apd, "%s\t%s\t%s", _username, remote, password);
		papchap->contents = g_slist_append( papchap->contents, apd );
}
