/* axstream.c
 *
 * Copyright (C) 2001 RedHat, Inc.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */
/*
 * Modified for SIKIGAMI 2001,2002 Axe,Inc.
 */


#include "axstream.h"
#include <sys/time.h>
#include <time.h>
#include <sys/types.h>
#include <errno.h>
#include <assert.h>
#include <malloc.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>

void
AxStream_init_valued( AxStream * stream, int _fd)
{
    AxStream_init( stream );
    stream->fd = _fd;
}

void
AxStream_init( AxStream * stream )
{
}


void
AxStream_clear( AxStream * stream )
{
		if ( 0 <= stream->fd ) {
    	AxStream_close( stream );
		}
}


void
AxStream_close( AxStream * stream)
{
    AxStream_flush( stream, 2000 );
    if ( 0 <= stream->fd ) close(stream->fd);
    stream->fd = -1;
}


void
AxStream_execute( AxStream * stream )
{
    // do nothing by default
}

int
AxStream_getfd( AxStream * stream )
{
    return stream->fd;
}

int
AxStream_isok( AxStream * stream )
{
    return ( stream->fd != -1);
}

size_t
AxStream_read( AxStream * stream, void *buf, size_t count)
{
 size_t st;

    st = read( stream->fd, buf, count );

		return st;
}

size_t
AxStream_write( AxStream * stream, void *buf, size_t count)
{
    if (!AxStream_isok(stream) || !buf || !count) return 0;

    return write( stream->fd, buf, count );
}

size_t
AxStream_print( AxStream * stream, char *buf)
{
    if (!AxStream_isok(stream) || !buf || !buf[0]) return 0;

    return write( stream->fd, buf, strlen(buf) );
}

size_t
AxStream_print1( AxStream * stream, char *buf, char *arg)
{
 char * msg;
 size_t ret;

    if (!AxStream_isok(stream) || !buf || !buf[0] || !arg ) return 0;

    msg = (char*)g_malloc(sizeof(char)*(strlen(buf)+strlen(arg)+1));

		sprintf( msg, buf, arg );

    ret = write( stream->fd, msg, strlen(msg) );

    free( msg );

    return ret;
}

void
AxStream_drain( AxStream * stream )
{
    char buf[1024];

    while(AxStream_select(stream,(time_t)0,1,0,0)) {
			AxStream_read(stream,buf,sizeof(buf));
	  }
}

void
AxStream_flush( AxStream * stream, time_t msec_timeout)
{
    if (!AxStream_isok(stream)) return;
    
	  AxStream_close( stream );
}


int
AxStream_select(AxStream * stream,
								time_t msec_timeout,
		      			int readable,
								int writable,
								int isexcept)
{
    int sure;
    int sel;
    struct timeval tv;
    SelectInfo si;
    
    if (!readable && !writable && !isexcept)
			return 0;  // why are you asking ME?
    
    if (!AxStream_isok(stream))
			return 0;

    FD_ZERO(&si.read);
    FD_ZERO(&si.write);
    FD_ZERO(&si.except);
    si.readable = readable;
    si.writable = writable;
    si.isexception = isexcept;
    si.max_fd = -1;
    si.msec_timeout = msec_timeout;
    
    sure = AxStream_select_setup(stream,&si);
    
    if (sure)
			tv.tv_sec = tv.tv_usec = 0; // never wait: already have a sure thing!
    else
    {
			tv.tv_sec = si.msec_timeout / 1000;
			tv.tv_usec = (si.msec_timeout % 1000) * 1000;
    }
    
    sel = select(si.max_fd+1, &si.read, &si.write, &si.except,
		   					 si.msec_timeout >= 0 ? &tv : (struct timeval*)NULL);
    
    if (sel < 0)
    {
			if (errno!=EAGAIN && errno!=EINTR && errno!=ENOBUFS)
	    	AxStream_seterr(stream,errno);
			return sure;
    }

    if (!sel) {
			return sure;	// timed out
		}
    
    return AxStream_isok(stream) && AxStream_test_set(stream,&si);
}

int
AxStream_select_setup(AxStream *stream,SelectInfo *si)
{
    int fd;

#if 0
    if (si->readable )
			return 1; // already ready
#endif
    
    fd = AxStream_getfd(stream);
    if (fd < 0) return 0;
    
    if (si->readable)
			FD_SET(fd, &si->read);
    if (si->writable )
			FD_SET(fd, &si->write);
    if (si->isexception)
			FD_SET(fd, &si->except);
    
    if (si->max_fd < fd)
			si->max_fd = fd;
    
    return 0;
}


int
AxStream_test_set( AxStream *stream,SelectInfo *si)
{
    if (	 AxStream_getfd(stream) >= 0
				&& FD_ISSET(AxStream_getfd(stream),&si->write) )
    {
			AxStream_flush(stream,0);
    }
    
    return		stream->fd >= 0
					&&  (FD_ISSET(AxStream_getfd(stream), &si->read)
	 				|| 	(FD_ISSET(AxStream_getfd(stream), &si->write) && si->writable)
	 				||   FD_ISSET(AxStream_getfd(stream), &si->except));
}
void
AxStream_seterr( AxStream * stream, int en )
{
	if ( ! stream->errnum ) {
		stream->errnum = en;
	}

	AxStream_close( stream );
}
void
AxStream_seterrstr( AxStream * stream, char * nouse )
{
	if ( ! stream->errnum ) {
		stream->errnum = -1;
	}

	AxStream_close( stream );
}
int
AxStream_geterr(AxStream*stream)
{
	return stream->errnum;
}
char *
AxStream_errstr(AxStream*stream)
{
	return strerror( stream->errnum );
}
