/* sgdialer.c
 * Copyright 2001 Axe, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Cambridge, MA
 * 02139, USA.
 *
 */


#include "axdialer.h"
#include "axconf.h"
#include "axstream.h"
#include "config.h"

#include <signal.h>
#include <sys/wait.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <syslog.h>

volatile int want_to_die = 0;

const char sgdialer_version_text[] =
"SgDialer 0.0.1"
": Copyright (c) 2001 Axe, Inc.\n";

const char sgdialer_help_text[] =
"Usage: sgdialer { option || sect1 sect2 sect3 ... } \n"
"\n"
"  options: --remotename  remotename to pass to pppd\n"
"     --help    display this help and exit\n"
"     --version output version information and exit\n"
"\n"
"Optional \"sect\" arguments refer to sections in /etc/sgdial/sgdial.conf.\n"
"Specified sections are all read, with later ones overriding previous ones.\n"
"Any options not in the listed sections are taken from [Dialer Defaults].\n"
"\n";


static void print_version()
{
    printf( "%s", sgdialer_version_text );
}

static void print_help()
{
    print_version();
    printf( "\n%s", sgdialer_help_text );
}

static void signalhandler( int parm )
{
    printf( "Caught signal #%d!  Attempting to exit gracefully...\n", parm );
    want_to_die = 0;
}


int main( int argc, char ** argv )
{
    AxDialer dialer;
    AxConf	cfg;
    int		i;
    int		ret = 1;
		int  chat_mode = 0;
    char * remote_name = NULL;
    GSList	* sections = NULL;
    char	* proname;
    AxConfigSection * sectptr;

    setuid( 0 );

//    openlog( argv[0],LOG_PID|LOG_CONSOLE,LOG_USER);

    AxConf_init_with_mode( & cfg, "/etc/sgdial/sgdial.conf", 0600 );

    signal( SIGTERM, signalhandler );
    signal( SIGINT, signalhandler );
    signal( SIGHUP, signalhandler );
    
    if( !AxConf_isok(&cfg) || !AxConf_isclean(&cfg) ) {
    	goto _END_;
    }
    
    if( argc > 1 ) {
    	for( i=1; i < argc; i++ ) {
            if( !strcmp( argv[i], "--remotename" ) ) {
                i++;
                remote_name = argv[i];
                continue;
            }
	    if( !strcmp( argv[i], "--help" ) ) {
	    	print_help();
	    	goto _END_;
	    }
	    else if( !strcmp( argv[i], "--version" ) ) {
	    	print_version();
	    	goto _END_;
	    }
			else if( !strcmp( argv[i], "--chat" ) ) {
			  chat_mode = 1;
				continue;
			}
	    else if( argv[i][0] == '-' ) {
		print_help();
	    	goto _END_;
	    }
	    proname = (char*)malloc(sizeof(char)*(strlen(argv[i])+8));
	    sprintf( proname, "Dialer %s", argv[i] );
	    sectptr = AxConf_at(&cfg,proname);
	    if ( sectptr ) {
	        sections = g_slist_append( sections, (gpointer)sectptr );
	    }
	    else {
	    }
	    free( proname );
    	}
    } else {
	sections = g_slist_append( sections, (gpointer)AxConf_at(&cfg,"Dialer Defaults") );
    }

    AxDialer_init_valued_parm( &dialer, &cfg, sections, chat_mode, remote_name );

    if( AxDialer_dial( &dialer ) == 0 ) {
	AxDialer_clear( & dialer );
   	goto _END_;
    }

    while( !want_to_die && AxDialer_isok(&dialer) 
	  && AxDialer_status(&dialer) != AxDialer_Idle ) {
	AxStream_select( (AxStream*)dialer.modem, 100, 1, 0, 0 );
	AxDialer_execute( &dialer );
	
	if (dialer.weird_pppd_problem) {
	    dialer.weird_pppd_problem = 0;
	}
    }

    AxDialer_hangup( & dialer );

    AxDialer_clear( & dialer );

    ret = 0;
_END_:
//    closelog( );

    return( ret );
}
